@@ -115,7 +115,7 @@ end
       # everything between it and us.
       self.class.state_sequence(self.retrieve, self.should).each do |state|
         method = state[direction]
-        raise Puppet::DevError, "Cannot move #{direction} from #{st[:name]}" unless method
+        raise Puppet::DevError, _("Cannot move %{direction} from %{name}") % { direction: direction, name: st[:name] } unless method
         provider_sync_send(method)
       end
 
@@ -262,7 +262,7 @@ end
   # Specify the sysidcfg file.  This is pretty hackish, because it's
   # only used to boot the zone the very first time.
   newparam(:sysidcfg) do
-    desc %{The text to go into the `sysidcfg` file when the zone is first
+    desc "The text to go into the `sysidcfg` file when the zone is first
       booted.  The best way is to use a template:
 
           # $confdir/modules/site/templates/sysidcfg.erb
@@ -290,7 +290,7 @@ end
           }
 
       The `sysidcfg` only matters on the first booting of the zone,
-      so Puppet only checks for it at that time.}
+      so Puppet only checks for it at that time."
   end
 
   newparam(:create_args) do
